# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: LGPL-3.0-or-later

"""API for the TextGrid aggregator service"""
import logging
from typing import List, Optional, Union, overload

import requests
from requests.models import Response

from tgclients.config import TextgridConfig

logger = logging.getLogger(__name__)


class Aggregator:
    """Provide access to the Textgrid Aggregator Service.
    API docs: https://textgridlab.org/doc/services/submodules/aggregator/docs/api.html"""

    def __init__(self, config: TextgridConfig = TextgridConfig()) -> None:
        self._url = config.aggregator
        self._config = config

    @overload
    def zip(self, textgrid_uris: str, sid: Optional[str] = None) -> Response:
        ...

    @overload
    def zip(self, textgrid_uris: List[str], sid: Optional[str] = None) -> Response:
        ...

    def zip(self, textgrid_uris: Union[str, List[str]], sid: Optional[str] = None) -> Response:
        """Download aggregated TextGrid objects as ZIP file.
        https://textgridlab.org/doc/services/submodules/aggregator/docs/zip.html

        Args:
            textgrid_uris (List[str] or str): a single or a list of TextGrid URIs
            sid (Optional[str], optional): Session ID. Defaults to None.

        Returns:
            Response: the response with zip file in body.content
        """
        if isinstance(textgrid_uris, list):
            textgrid_uris = ','.join(textgrid_uris)
        url = self._url + '/zip/'
        response = requests.get(url + textgrid_uris, { 'sid': sid },
                                timeout=self._config.http_timeout)
        return response

    @overload
    def text(self, textgrid_uris: str, sid: Optional[str] = None) -> Response:
        ...

    @overload
    def text(self, textgrid_uris: List[str], sid: Optional[str] = None) -> Response:
        ...

    # python 3.10 allows writinh Union as |
    # https://www.blog.pythonlibrary.org/2021/09/11/python-3-10-simplifies-unions-in-type-annotations/
    def text(self, textgrid_uris: Union[str, List[str]], sid: Optional[str] = None) -> Response:
        """Download aggregated TextGrid objects as plain text file.

        Args:
            textgrid_uris (List[str] or str): a single or a list of TextGrid URIs
            sid (Optional[str], optional): Session ID. Defaults to None.

        Returns:
            Response: the respone with the text in the body
        """
        if isinstance(textgrid_uris, list):
            textgrid_uris = ','.join(textgrid_uris)
        url = self._url + '/text/'
        response = requests.get(url + textgrid_uris, { 'sid': sid },
                                timeout=self._config.http_timeout)
        return response

    @overload
    def teicorpus(self, textgrid_uris: str, sid: Optional[str] = None) -> Response:
        ...

    @overload
    def teicorpus(self, textgrid_uris: List[str], sid: Optional[str] = None) -> Response:
        ...

    def teicorpus(self, textgrid_uris: Union[str, List[str]], sid: Optional[str] = None) -> Response:
        """Download aggregated TextGrid objects as TEI corpus.

        Args:
            textgrid_uris (List[str] or str): a single or a list of TextGrid URIs
            sid (Optional[str], optional): Session ID. Defaults to None.

        Returns:
            Response: the respone with the TEI corpus in the body
        """
        if isinstance(textgrid_uris, list):
            textgrid_uris = ','.join(textgrid_uris)
        url = self._url + '/teicorpus/'
        response = requests.get(url + textgrid_uris, { 'sid': sid },
                                timeout=self._config.http_timeout)
        return response



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: LGPL-3.0-or-later

"""Helper functions to work with TextGrid metadata XML"""
import io
import os
import re
import logging
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import Optional

import defusedxml.ElementTree as dT
from jinja2 import Environment, FileSystemLoader
from xsdata.formats.dataclass.context import XmlContext
from xsdata.formats.dataclass.parsers import XmlParser

from tgclients.databinding.tgsearch import Response, ResultType
from tgclients.databinding.textgrid_metadata_2010 import MetadataContainerType

try:
    import icu
except ImportError:
    icu = None

logger = logging.getLogger(__name__)

__location__ = os.path.realpath(
    os.path.join(os.getcwd(), os.path.dirname(__file__)))

class TextgridMetadata:
    """Helper functions to work with TextGrid metadata XML"""

    def __init__(self):
        context = XmlContext()
        self._parser = XmlParser(context=context)
        self._file_extension_map = self._build_extension_map()
        if icu is not None:
            self._transliterator = self._create_transliterator()

    @staticmethod
    def create(title: str, mimetype: str) -> str:
        """Create XML metadata for an TextGrid Object

        Args:
            title (str): title of the object
            mimetype (str): format / MIME type of the object

        Returns:
            str: XML metadata as string
        """
        path = Path(__file__).parent / 'templates'
        env = Environment(
            loader=FileSystemLoader(Path(path)), autoescape=True)
        template = env.get_template('metadata.xml.jinja2')
        metadata = template.render(title=title, format=mimetype)
        return metadata

    def build(self, title: str, mimetype: str) -> MetadataContainerType:
        """Build metadata for an TextGrid Object

        Args:
            title (str): title of the object
            mimetype (str): format / MIME type of the object

        Returns:
            MetadataContainerType: metadata
        """
        metadata = TextgridMetadata().create(title, mimetype)
        return self._parser.from_string(metadata, MetadataContainerType)

    @staticmethod
    def get_xpath(xml: str, xpath: str) -> str:
        namespaces = TextgridMetadata._register_namespaces(xml)
        xml_stream = io.StringIO(xml)
        element_tree = dT.parse(xml_stream)
        result = element_tree.find(xpath, namespaces).text
        return result

    @staticmethod
    @DeprecationWarning
    # This is not used. TODO: remove
    def get_xpath2(xml: str, xpath: str) -> str:
        namespaces = TextgridMetadata._register_namespaces(xml)
        xml_stream = io.StringIO(xml)
        element_tree = dT.parse(xml_stream)
        result = element_tree.findall(xpath, namespaces)
        return result

    @staticmethod
    def _register_namespaces(xml: str) -> dict:
        """Register namespaces to the global ElementTree and return a
        dictionary of them.

        Args:
            xml (str): XML Document

        Returns:
            dict: Namespaces dictionary
        """
        xml_stream = io.StringIO(xml)
        # Uses a list comprehension and element tree's iterparse function to
        # create a dictionary containing the namespace prefix and its uri. The
        # underscore is utilized to remove the "start-ns" output. `iterparse`
        # returns an iterator providing (event, elem) pairs.
        # see https://medium.datadriveninvestor.com/getting-started-using-pythons-elementtree-to-navigate-xml-files-dc9bc720eaa6
        namespaces = {node[0]: node[1]
                      for _, node in dT.iterparse(xml_stream, events=['start-ns'])}

        # Iterates through the newly created namespace dictionary registering the prefixes
        for prefix, uri in namespaces.items():
            ET.register_namespace(prefix + '_', uri)

        return namespaces

    def searchresponse2object(self, xml: str) -> Response:
        return self._parser.from_string(xml, Response)

    def filename_from_metadata(self, metadata: ResultType) -> str:
        """Generate a filename for a textgrid search metadata result
        containg title, textfgrid-uri and extension

        Args:
            metadata (ResultType): tgsearch metadata result

        Returns:
            str: the filename
        """
        title = metadata.object_value.generic.provided.title[0]
        mimetype = metadata.object_value.generic.provided.format
        uri = metadata.object_value.generic.generated.textgrid_uri.value
        return self.filename(title, uri, mimetype)

    def filename(self, title: str, tguri: str, mimetype: str) -> str:
        """Generate a filename for the triple of
        title, textfgrid-uri and extension

        Args:
            title (str): the title
            tguri (str): the textgrid uri
            mimetype (str): the mime type (e.g. 'text/xml')

        Returns:
            str: the filename
        """
        title = self.transliterate(title)
        tg_id = self.remove_tg_prefix(tguri)
        ext = self.extension_for_format(mimetype)
        if ext is not None:
            return f'{title}.{tg_id}.{ext}'
        else:
            return f'{title}.{tg_id}'

    def _build_extension_map(self) -> dict:
        # converted to python from info.textgrid.utils.export.filenames.FileExtensionMap
        # of link-rewriter (https://gitlab.gwdg.de/dariah-de/textgridrep/link-rewriter)
        extension_map = {}
        map_line_pattern = re.compile(
            '^[ \t]*([^# \t]+)[ \t]*([^#]+)[ \t]*(#.*)?$')
        space_pattern = re.compile('[ \t]+')

        with open(os.path.join(__location__, 'mime.types'), encoding='utf8') as mimefile:
            for line in mimefile.readlines():
                line_match = map_line_pattern.match(line.rstrip('\n'))
                if line_match is not None:
                    entry = space_pattern.split(line_match.group(2))
                    # extend the list in the dict, so extensions definded first are first in list
                    if line_match.group(1) not in extension_map:
                        extension_map[line_match.group(1)] = entry
                    else:
                        extension_map[line_match.group(1)].extend(entry)

        return extension_map

    def extension_for_format(self, mimetype: str) -> Optional[str]:
        """Find a matching extension for a textgrid mime type.
        The first matching extension for a mime type is returned, so
        extensions defined first in mime.types will be used.

        Args:
            mimetype (str): the mime type, as found in textgrid
                            metadata format field (e.g. text/xml)

        Returns:
            str: a filename extension
        """
        if mimetype in self._file_extension_map:
            return self._file_extension_map[mimetype][0]
        else:
            return None

    @staticmethod
    def remove_tg_prefix(tguri: str) -> str:
        return tguri[9:]

    @staticmethod
    def id_from_filename(filename: str) -> str:
        """extract the id from a filename which is named according to link rewriters
        textgrid metadata to filename mapping

        Args:
            filename (str): the filename

        Returns:
            str: the id
        """
        last_dot = filename.rfind('.')
        next_to_last_dot = filename.rfind('.', 0, last_dot)
        # a textgrid uri has a revision number in the end.
        # if the chars after the last dot are not numeric, we have a filename extension
        if not filename[last_dot+1:].isnumeric():
            # extension is there? we need the '.' before the dot separating the uri
            # from the revision component
            next_to_last_dot = filename.rfind('.', 0, next_to_last_dot)
        else:
            # there is no extension to cut of, we want the end of the string
            last_dot = None

        return filename[next_to_last_dot+1:last_dot]

    def transliterate(self, title: str) -> str:
        """replace all chars which may be problematic in filenames from title string

        Args:
            title (str): a title from textgrid metadata

        Returns:
            str: the title string with problematic chars replaced
        """
        name: str = ''
        if icu is None:
            logger.warning('Transliterating without PyICU, you may need that for correct results')
            name = title.replace(' ', '_').replace(']', '_').replace('[', '_').replace(':', '_')
        else:
            name = self._transliterator.transliterate(title)
        return name

    # return type needs to be hidden, because it is not available if PyICU is not installed
    def _create_transliterator(self):
        with open(os.path.join(__location__, 'tgfilenames.rules'), encoding='utf8') as rulesfile:
            rules = rulesfile.read()
            return icu.Transliterator.createFromRules(
                'TgFilenames', rules, icu.UTransDirection.FORWARD)



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: CC0-1.0

from enum import Enum

__NAMESPACE__ = "http://textgrid.info/namespaces/metadata/script/2010"


class FormOfNotationType(Enum):
    """Set of characters and/or symbols used to express the content of a
    resource.

    Same as: http://rdvocab.info/Elements/formOfNotation.

    Attributes
        ARAB: Arabic
        ARMI: Imperial Aramaic
        ARMN: Armenian
        AVST: Avestan
        BALI: Balinese
        BAMU: Bamum
        BASS: Bassa Vah
        BATK: Batak
        BENG: Bengali
        BLIS: Blissymbols
        BOPO: Bopomofo
        BRAH: Brahmi
        BRAI: Braille
        BUGI: Buginese
        BUHD: Buhid
        CAKM: Chakma
        CANS: Unified Canadian Aboriginal Syllabics
        CARI: Carian
        CHAM: Cham
        CHER: Cherokee
        CIRT: Cirth
        COPT: Coptic
        CPRT: Cypriot
        CYRL: Cyrillic
        CYRS: Cyrillic (Old Church Slavonic variant)
        DEVA: Devanagari (Nagari)
        DSRT: Deseret (Mormon)
        EGYD: Egyptian demotic
        EGYH: Egyptian hieratic
        EGYP: Egyptian hieroglyphs
        ETHI: Ethiopic (Geʻez)
        GEOR: Georgian (Mkhedruli)
        GEOK: Khutsuri (Asomtavruli and Nuskhuri)
        GLAG: Glagolitic
        GOTH: Gothic
        GRAN: Grantha
        GREK: Greek
        GUJR: Gujarati
        GURU: Gurmukhi
        HANG: Hangul (Hangŭl, Hangeul)
        HANI: Han (Hanzi, Kanji, Hanja)
        HANO: Hanunoo (Hanunóo)
        HANS: Han (Simplified variant)
        HANT: Han (Traditional variant)
        HEBR: Hebrew
        HIRA: Hiragana
        HMNG: Pahawh Hmong
        HRKT: (alias for Hiragana + Katakana)
        HUNG: Old Hungarian
        INDS: Indus (Harappan)
        ITAL: Old Italic (Etruscan, Oscan, etc.)
        JAVA: Javanese
        JPAN: Japanese (alias for Han + Hiragana + Katakana)
        KALI: Kayah Li
        KANA: Katakana
        KHAR: Kharoshthi
        KHMR: Khmer
        KNDA: Kannada
        KORE: Korean (alias for Hangul + Han)
        KPEL: Kpelle
        KTHI: Kaithi
        LANA: Tai Tham (Lanna)
        LAOO: Lao
        LATF: Latin (Fraktur variant)
        LATG: Latin (Gaelic variant)
        LATN: Latin
        LEPC: Lepcha (Róng)
        LIMB: Limbu
        LINA: Linear A
        LINB: Linear B
        LISU: Lisu (Fraser)
        LOMA: Loma
        LYCI: Lycian
        LYDI: Lydian
        MAND: Mandaic, Mandaean
        MANI: Manichaean
        MAYA: Mayan hieroglyphs
        MEND: Mende
        MERC: Meroitic Cursive
        MERO: Meroitic Hieroglyphs
        MLYM: Malayalam
        MOON: Moon (Moon code, Moon script, Moon type)
        MONG: Mongolian
        MTEI: Meitei Mayek (Meithei, Meetei)
        MYMR: Myanmar (Burmese)
        NARB: Old North Arabian (Ancient North Arabian)
        NBAT: Nabataean
        NKGB: Nakhi Geba ('Na-'Khi ²Ggŏ-¹baw, Naxi Geba)
        NKOO: N’Ko
        OGAM: Ogham
        OLCK: Ol Chiki (Ol Cemet’, Ol, Santali)
        ORKH: Old Turkic, Orkhon Runic
        ORYA: Oriya
        OSMA: Osmanya
        PALM: Palmyrene
        PERM: Old Permic
        PHAG: Phags-pa
        PHLI: Inscriptional Pahlavi
        PHLP: Psalter Pahlavi
        PHLV: Book Pahlavi
        PHNX: Phoenician
        PLRD: Miao (Pollard)
        PRTI: Inscriptional Parthian
        QAAA: Reserved for private use (start)
        QABX: Reserved for private use (end)
        RJNG: Rejang (Redjang, Kaganga)
        RORO: Rongorongo
        RUNR: Runic
        SAMR: Samaritan
        SARA: Sarati
        SARB: Old South Arabian
        SAUR: Saurashtra
        SGNW: SignWriting
        SHAW: Shavian (Shaw)
        SINH: Sinhala
        SUND: Sundanese
        SYLO: Syloti Nagri
        SYRC: Syriac
        SYRE: Syriac (Estrangelo variant)
        SYRJ: Syriac (Western variant)
        SYRN: Syriac (Eastern variant)
        TAGB: Tagbanwa
        TALE: Tai Le
        TALU: New Tai Lue
        TAML: Tamil
        TAVT: Tai Viet
        TELU: Telugu
        TENG: Tengwar
        TFNG: Tifinagh (Berber)
        TGLG: Tagalog (Baybayin, Alibata)
        THAA: Thaana
        THAI: Thai
        TIBT: Tibetan
        UGAR: Ugaritic
        VAII: Vai
        VISP: Visible Speech
        WARA: Warang Citi (Varang Kshiti)
        XPEO: Old Persian
        XSUX: Cuneiform, Sumero-Akkadian
        YIII: Yi
        ZINH: Code for inherited script
        ZMTH: Mathematical notation
        ZSYM: Symbols
        ZXXX: Code for unwritten documents
        ZYYY: Code for undetermined script
        ZZZZ: Code for uncoded script
    """
    ARAB = "Arab"
    ARMI = "Armi"
    ARMN = "Armn"
    AVST = "Avst"
    BALI = "Bali"
    BAMU = "Bamu"
    BASS = "Bass"
    BATK = "Batk"
    BENG = "Beng"
    BLIS = "Blis"
    BOPO = "Bopo"
    BRAH = "Brah"
    BRAI = "Brai"
    BUGI = "Bugi"
    BUHD = "Buhd"
    CAKM = "Cakm"
    CANS = "Cans"
    CARI = "Cari"
    CHAM = "Cham"
    CHER = "Cher"
    CIRT = "Cirt"
    COPT = "Copt"
    CPRT = "Cprt"
    CYRL = "Cyrl"
    CYRS = "Cyrs"
    DEVA = "Deva"
    DSRT = "Dsrt"
    EGYD = "Egyd"
    EGYH = "Egyh"
    EGYP = "Egyp"
    ETHI = "Ethi"
    GEOR = "Geor"
    GEOK = "Geok"
    GLAG = "Glag"
    GOTH = "Goth"
    GRAN = "Gran"
    GREK = "Grek"
    GUJR = "Gujr"
    GURU = "Guru"
    HANG = "Hang"
    HANI = "Hani"
    HANO = "Hano"
    HANS = "Hans"
    HANT = "Hant"
    HEBR = "Hebr"
    HIRA = "Hira"
    HMNG = "Hmng"
    HRKT = "Hrkt"
    HUNG = "Hung"
    INDS = "Inds"
    ITAL = "Ital"
    JAVA = "Java"
    JPAN = "Jpan"
    KALI = "Kali"
    KANA = "Kana"
    KHAR = "Khar"
    KHMR = "Khmr"
    KNDA = "Knda"
    KORE = "Kore"
    KPEL = "Kpel"
    KTHI = "Kthi"
    LANA = "Lana"
    LAOO = "Laoo"
    LATF = "Latf"
    LATG = "Latg"
    LATN = "Latn"
    LEPC = "Lepc"
    LIMB = "Limb"
    LINA = "Lina"
    LINB = "Linb"
    LISU = "Lisu"
    LOMA = "Loma"
    LYCI = "Lyci"
    LYDI = "Lydi"
    MAND = "Mand"
    MANI = "Mani"
    MAYA = "Maya"
    MEND = "Mend"
    MERC = "Merc"
    MERO = "Mero"
    MLYM = "Mlym"
    MOON = "Moon"
    MONG = "Mong"
    MTEI = "Mtei"
    MYMR = "Mymr"
    NARB = "Narb"
    NBAT = "Nbat"
    NKGB = "Nkgb"
    NKOO = "Nkoo"
    OGAM = "Ogam"
    OLCK = "Olck"
    ORKH = "Orkh"
    ORYA = "Orya"
    OSMA = "Osma"
    PALM = "Palm"
    PERM = "Perm"
    PHAG = "Phag"
    PHLI = "Phli"
    PHLP = "Phlp"
    PHLV = "Phlv"
    PHNX = "Phnx"
    PLRD = "Plrd"
    PRTI = "Prti"
    QAAA = "Qaaa"
    QABX = "Qabx"
    RJNG = "Rjng"
    RORO = "Roro"
    RUNR = "Runr"
    SAMR = "Samr"
    SARA = "Sara"
    SARB = "Sarb"
    SAUR = "Saur"
    SGNW = "Sgnw"
    SHAW = "Shaw"
    SINH = "Sinh"
    SUND = "Sund"
    SYLO = "Sylo"
    SYRC = "Syrc"
    SYRE = "Syre"
    SYRJ = "Syrj"
    SYRN = "Syrn"
    TAGB = "Tagb"
    TALE = "Tale"
    TALU = "Talu"
    TAML = "Taml"
    TAVT = "Tavt"
    TELU = "Telu"
    TENG = "Teng"
    TFNG = "Tfng"
    TGLG = "Tglg"
    THAA = "Thaa"
    THAI = "Thai"
    TIBT = "Tibt"
    UGAR = "Ugar"
    VAII = "Vaii"
    VISP = "Visp"
    WARA = "Wara"
    XPEO = "Xpeo"
    XSUX = "Xsux"
    YIII = "Yiii"
    ZINH = "Zinh"
    ZMTH = "Zmth"
    ZSYM = "Zsym"
    ZXXX = "Zxxx"
    ZYYY = "Zyyy"
    ZZZZ = "Zzzz"



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: CC0-1.0

from dataclasses import dataclass, field
from typing import List

__NAMESPACE__ = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"


@dataclass
class RdfType:
    """
    Content may be in any namespace (namespace=##any, see
    http://www.w3.org/TR/xmlschema-1/#declare-openness) and the elements are
    only validated if a schema is available that uniquely determines the
    declaration (processContents="lax", see
    http://www.w3.org/TR/xmlschema-1/#declare-openness)
    """
    class Meta:
        name = "rdfType"

    any_element: List[object] = field(
        default_factory=list,
        metadata={
            "type": "Wildcard",
            "namespace": "##any",
        }
    )


@dataclass
class Rdf(RdfType):
    class Meta:
        name = "RDF"
        namespace = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: LGPL-3.0-or-later

"""Provide access to the TextGrid Authorization Service."""
import logging
from typing import Optional, List

from zeep import Client
from zeep.exceptions import Fault, TransportError

from tgclients.config import TextgridConfig

TG_STANDARD_ROLE_MANAGER = 'Projektleiter'
TG_STANDARD_ROLE_ADMINISTRATOR = 'Administrator'
TG_STANDARD_ROLE_EDITOR = 'Bearbeiter'
TG_STANDARD_ROLE_OBSERVER = 'Beobachter'

logger = logging.getLogger(__name__)
# prevent the zeep "Forcing soap:address location to HTTPS" warning
logging.getLogger('zeep.wsdl.bindings.soap').setLevel(logging.ERROR)


class TextgridAuth:
    """Provide access to the TextGrid Authorization Service."""

    def __init__(self, config: TextgridConfig = TextgridConfig()) -> None:
        self._config = config
        self._client = self._connect()
        self._extra_crud_client = self._connect_extra_crud()

    def _connect(self) -> Client:
        """Internal helper that provides a SOAP client that is configured for
        the use with the Textgrid Auth service.

        Returns:
            Client: A SOAP client
        """
        client = Client(self._config.auth_wsdl)
        # this is a dirty hack; should be remediated
        client.service._binding_options['address'] = self._config.auth_address
        return client

    def _connect_extra_crud(self) -> Client:
        """Internal helper that provides a SOAP client that is configured for
        the use with the Textgrid Auth service (the extra crud service).

        Returns:
            Client: A SOAP client
        """
        client = Client(self._config.extra_crud_wsdl)
        # this is a dirty hack; should be remediated
        client.service._binding_options['address'] = self._config.extra_crud_address
        return client

    def list_assigned_projects(self, sid: str) -> List[str]:
        """Get assigned projects.

        Args:
            sid (str): Session ID

        Raises:
            TextGridAuthException: in case of transport exceptions

        Returns:
            List: A list of project id strings
        """
        try:
            return self._client.service.tgAssignedProjects(sid)
        except TransportError as error:
            message = 'Error listing assigned projects. Is your sessionID valid?'
            logger.warning(message)
            raise TextGridAuthException(message) from error

    def list_all_projects(self) -> List[str]:
        """Returns all projects stored in this RBAC instance with ID, name,
        and description. See also getProjectDescription(). SID
        is not needed as this information can be reviewed publicly.

        Returns:
            list: list of each project with ID, name and description
        """
        # sid is optional, so we pass empty sid
        return self._client.service.getAllProjects('')

    def get_project_description(self, project_id: str):
        """Returns name and description of project identified by ID. See also getAllProjects().

        Args:
            project_id (str): the project ID

        Returns:
            zeep.objects.projectInfo: project info with id, name and description
        """
        return self._client.service.getProjectDescription('', '', project_id)

    def create_project(self, sid: str, name: str, description: str,
                       default_owner_roles: Optional[bool] = True) -> str:
        """Create a new project

        Args:
            sid (str): TextGrid Session ID
            name (str): name of the project
            description (str): description for the project
            default_owner_roles (bool, optional): whether to assign the default roles to the owner
                                                  (editor, authority to delete). Defaults to True.

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            str: the project ID of the created project
        """
        try:
            project_id = self._client.service.createProject(auth=sid, name=name,
                                                            description=description)
        except Fault as fault:
            message = 'Error creating project. Is your sessionID valid?'
            logger.warning(message)
            raise TextGridAuthException(message) from fault

        if default_owner_roles:
            eppn = self.get_eppn_for_sid(sid)
            self.add_editor_to_project(sid, project_id, eppn)
            self.add_admin_to_project(sid, project_id, eppn)

        return project_id

    def get_eppn_for_sid(self, sid: str) -> str:
        """Get the EPPN belonging to a sessionID

        Args:
            sid (str): TextGrid Session ID

        Raises:
            TextGridAuthException: in case of transport exceptions

        Returns:
            str: the EPPN
        """
        try:
            eppn = self._extra_crud_client.service.getEPPN(auth=sid, secret='')
        except TransportError as error:
            message = 'Error getting eppn. Is your sessionID valid?'
            logger.warning(message)
            raise TextGridAuthException(message) from error
        return eppn

    def delete_project(self, sid: str, project_id: str) -> bool:
        """Delete a project

        Args:
            sid (str): TextGrid Session ID
            project_id (str): the project ID

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            bool: true in case of success
        """
        try:
            status = self._client.service.deleteProject(
                auth=sid, project=project_id)
        except Fault as fault:
            message = 'Error deleting project. Is your sessionID valid?'
            logger.warning(message)
            raise TextGridAuthException(message) from fault
        return status

    def add_admin_to_project(self, sid: str, project_id: str, eppn: str) -> bool:
        """Give an user the admin role in a project

        Args:
            sid (str): TextGrid Session ID
            project_id (str): the project ID
            eppn (str): the eppn identifying the user

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            bool: true in case of success
        """
        try:
            return self._add_role_to_project(sid, project_id, eppn, TG_STANDARD_ROLE_ADMINISTRATOR)
        except TextGridAuthException as exception:
            raise exception

    def add_editor_to_project(self, sid: str, project_id: str, eppn: str) -> bool:
        """Give an user the editor role in a project

        Args:
            sid (str): TextGrid Session ID
            project_id (str): the project ID
            eppn (str): the eppn identifying the user

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            bool: true in case of success
        """
        try:
            return self._add_role_to_project(sid, project_id, eppn, TG_STANDARD_ROLE_EDITOR)
        except TextGridAuthException as exception:
            raise exception

    def add_manager_to_project(self, sid: str, project_id: str, eppn: str) -> bool:
        """Give an user the manager role in a project

        Args:
            sid (str): TextGrid Session ID
            project_id (str): the project ID
            eppn (str): the eppn identifying the user

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            bool: true in case of success
        """
        try:
            return self._add_role_to_project(sid, project_id, eppn, TG_STANDARD_ROLE_MANAGER)
        except TextGridAuthException as exception:
            raise exception

    def add_observer_to_project(self, sid: str, project_id: str, eppn: str) -> bool:
        """Give an user the observer role in a project

        Args:
            sid (str): TextGrid Session ID
            project_id (str): the project ID
            eppn (str): the eppn identifying the user

        Raises:
            TextGridAuthException: in case of tgauth faults

        Returns:
            bool: true in case of success
        """
        try:
            return self._add_role_to_project(sid, project_id, eppn, TG_STANDARD_ROLE_OBSERVER)
        except TextGridAuthException as exception:
            raise exception

    def _add_role_to_project(self, sid: str, project_id: str, eppn: str, role: str) -> bool:
        rolename = role + ',' + project_id + ',Projekt-Teilnehmer'
        try:
            status = self._client.service.addMember(
                auth=sid, username=eppn, role=rolename)
        except Fault as fault:
            message = 'Error adding role to project. Is your sessionID valid?'
            logger.warning(message)
            raise TextGridAuthException(message) from fault
        return status


class TextGridAuthException(Exception):
    """Exception communicating with tgauth"""



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: CC0-1.0

"""Variable config options with defaults to be used with the TextGrid clients library."""
import logging
from typing import Optional

logger = logging.getLogger(__name__)

DEFAULT_TIMEOUT: float = 120.00

class TextgridConfig:
    """Provide Standard configuration / URLs for TextGrid Services
    """

    def __init__(self, host: Optional[str] = 'https://textgridlab.org') -> None:
        """TextGrid Service Endpoint Configuration.
        Default to production system. Overwrite the host string to
        connect to other systems, e.g.:
        * Test-System: https://test.textgridlab.org
        * Development-System: https://dev.textgridlab.org

        Args:
            host (Optional[str]): [TextGrid system]. Defaults to 'https://textgridlab.org'.
        """
        if host.endswith('/'):
            logger.warning('trailing slash in hostname detected and removed')
            host = host[:-1]
        self._host = host
        self._http_timeout = DEFAULT_TIMEOUT

    @property
    def host(self) -> str:
        """the host URL

        Returns:
            str: the configured host URL
        """
        return self._host

    @property
    def auth_wsdl(self) -> str:
        """the tgauth wsdl location

        Returns:
            str: the tgauth wsdl location
        """
        return self._host + '/1.0/tgauth/wsdl/tgextra.wsdl'

    @property
    def auth_address(self) -> str:
        """the tgauth service location

        Returns:
            str: the tgauth service location
        """
        return self._host + '/1.0/tgauth/tgextra.php'

    @property
    def extra_crud_wsdl(self) -> str:
        """the tgextra wsdl location

        Returns:
            str: the tgextra wsdl location
        """
        return self._host + '/1.0/tgauth/wsdl/tgextra-crud.wsdl'

    @property
    def extra_crud_address(self) -> str:
        """the tgextra service location

        Returns:
            str: the tgextra service location
        """
        return self._host + '/1.0/tgauth/tgextra-crud.php'

    @property
    def search(self) -> str:
        """the nonpublic tgsearch service location

        Returns:
            str: the nonpublic tgsearch service location
        """
        return self._host + '/1.0/tgsearch'

    @property
    def search_public(self) -> str:
        """the public tgsearch service location

        Returns:
            str: the public tgsearch service location
        """
        return self._host + '/1.0/tgsearch-public'

    @property
    def crud(self) -> str:
        """the nonpublic tgcrud REST service location

        Returns:
            str: the nonpublic tgcrud REST service location
        """
        return self._host + '/1.0/tgcrud/rest'

    @property
    def crud_public(self) -> str:
        """the public tgcrud REST service location

        Returns:
            str: the public tgcrud REST service location
        """
        return self._host + '/1.0/tgcrud-public/rest'

    @property
    def aggregator(self) -> str:
        """the aggregator service location

        Returns:
            str: the aggregator service location
        """
        return self._host + '/1.0/aggregator'

    @property
    def http_timeout(self) -> float:
        """HTTP timeout to be used when accessing TextGrid services

        Returns:
            float: http timeout in seconds
        """
        return self._http_timeout

    @http_timeout.setter
    def http_timeout(self, value: float) -> None:
        """Set HTTP timeout to be used when accessing TextGrid services

        Args:
            value (float): the timeout
        """
        self._http_timeout = value



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: CC0-1.0

# This file is only required by pip-tools and older versions of pip now
from setuptools import setup

setup()



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: LGPL-3.0-or-later

"""TextGrid CRUD API."""
import logging
from io import BytesIO
from typing import Optional

import requests
from requests.models import Response
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
from xsdata.formats.dataclass.context import XmlContext
from xsdata.formats.dataclass.parsers import XmlParser
from xsdata.formats.dataclass.serializers import XmlSerializer

from tgclients.config import TextgridConfig
from tgclients.databinding.textgrid_metadata_2010 import MetadataContainerType

logger = logging.getLogger(__name__)
RESPONSE_ENCODING = 'utf-8'


class TextgridCrudRequest:
    """Provide low level access to the TextGrid CRUD Service."""

    def __init__(self, url: str = TextgridConfig().crud_public,
                 config: TextgridConfig = TextgridConfig()) -> None:
        self._url = url
        self._config = config

    def read_data(self, textgrid_uri: str, sid: Optional[str] = None) -> Response:
        """Read Data

        Args:
            textgrid_uri (str): Textgrid URI
            sid (Optional[str]): Session ID. Defaults to None.

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            Response: HTTP response from service
        """
        # defer downloading the response body until accessing Response.content
        response = requests.get(self._url + '/' + textgrid_uri + '/data',
                                { 'sessionId':  sid },
                                stream=True,
                                timeout=self._config.http_timeout)
        return self._handle_response(response)

    def read_metadata(self, textgrid_uri: str, sid: Optional[str] = None) -> Response:
        """Read Metadata

        Args:
            textgrid_uri (str): Textgrid URI
            sid (Optional[str]): Session ID. Defaults to None.

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            Response: HTTP response from service
        """
        response = requests.get(self._url + '/' + textgrid_uri + '/metadata',
                                { 'sessionId':  sid },
                                stream=True,
                                timeout=self._config.http_timeout)
        return self._handle_response(response)

    def create_resource(self, sid: str, project_id: str, data, metadata) -> Response:
        """Create a TextGrid object

        Args:
            sid (str): Session ID
            project_id (str): Project ID
            data ([type]): the data
            metadata ([type]): the metadata

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            Response: HTTP response from service with metadata from newly created object
        """
        encoder = self._prepare_multipart(metadata, data)
        params = {'sessionId': sid, 'projectId': project_id,
                  'createRevision': 'false'}
        response = requests.post(
            self._url + '/' + 'create', params=params, data=encoder,
            headers={'Content-Type': encoder.content_type},
            timeout=self._config.http_timeout)
        return self._handle_response(response)

    def update_resource(self, sid: str, textgrid_uri: str, data, metadata) -> Response:
        """Update a TextGrid object

        Args:
            sid (str): Session ID
            textgrid_uri (str): Textgrid URI
            data ([type]): the data
            metadata ([type]): the metadata

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            Response: HTTP response from service with updated metadata
        """
        encoder = self._prepare_multipart(metadata, data)
        params = {'sessionId': sid}
        response = requests.post(
            self._url + '/' + textgrid_uri + '/update', params=params, data=encoder,
            headers={'Content-Type': encoder.content_type},
            timeout=self._config.http_timeout)
        return self._handle_response(response)

    def update_metadata(self, sid: str, textgrid_uri: str, metadata) -> Response:
        """Update metadata for TextGrid object

        Args:
            sid (str): Session ID
            textgrid_uri (str): Textgrid URI
            metadata ([type]): the metadata

        Returns:
            Response: HTTP response from service with updated metadata
        """
        encoder = self._prepare_multipart(metadata)
        params = {'sessionId': sid}
        response = requests.post(
            self._url + '/' + textgrid_uri + '/updateMetadata', params=params, data=encoder,
            headers = {'Content-Type': encoder.content_type},
            timeout = self._config.http_timeout)
        return self._handle_response(response)

    def delete_resource(self, sid: str, textgrid_uri: str) -> Response:
        """Delete a TextGrid object

        Args:
            sid (str): Session ID
            textgrid_uri (str): Textgrid URI

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            Response: HTTP response from service
        """
        params = {'sessionId': sid}
        response = requests.get(
            self._url + '/' + textgrid_uri + '/delete', params=params,
            timeout=self._config.http_timeout)
        return self._handle_response(response)

    @staticmethod
    def _handle_response(response: Response) -> Response:
        """Error handling for responses from crud

        Args:
            response (Response): a response from tgcrud

        Raises:
            TextgridCrudException: if HTTP status code >= 400

        Returns:
            Response: the response
        """
        response.encoding = RESPONSE_ENCODING
        if not response.ok:
            message = '[Error] HTTP Code: ' + \
                str(response.status_code) + ' - ' + response.text[0:255]
            logger.warning(message)
            raise TextgridCrudException(message)
        return response

    @staticmethod
    def _prepare_multipart(metadata, data=None):
        """Create a streaming multipart object.
        Monitor the upload progress if log level is DEBUG.

        See also: https://toolbelt.readthedocs.io/en/latest/uploading-data.html

        Args:
            metadata ([type]): the metadata
            data ([type]): the data

        Returns:
            [MultipartEncoder]: Multipart containing data and metadata
        """
        fields = {
            'tgObjectMetadata': ('tgObjectMetadata', metadata, 'text/xml')
        }
        if data:
            fields['tgObjectData'] = (
                'tgObjectData', data, 'application/octet-stream')

        encoder = MultipartEncoder(fields=fields)
        if logger.isEnabledFor(logging.DEBUG):
            return MultipartEncoderMonitor(encoder, TextgridCrudRequest._debug_monitor_callback)

        return encoder

    @staticmethod
    def _debug_monitor_callback(monitor: MultipartEncoderMonitor):
        """Callback for _prepare_multipart.
        Helper to log upload progress for streaming multipart when log level is DEBUG.

        Args:
            monitor (MultipartEncoderMonitor): the monitor
        """
        logger.debug('[debug multipart upload] bytes read: %s ',
                     monitor.bytes_read)


class TextgridCrudException(Exception):
    """Exception communicating with tgcrud"""


class TextgridCrud(TextgridCrudRequest):
    """Provide access to the Textgrid CRUD Service using a XML data binding """

    def __init__(self, url: str = TextgridConfig().crud_public) -> None:
        super().__init__(url)
        # It’s recommended to either reuse the same parser/serializer instance
        # or reuse the context instance. see https://xsdata.readthedocs.io/en/latest/xml.html
        context = XmlContext()
        self._parser = XmlParser(context=context)
        self._serializer = XmlSerializer()

    def create_resource(self, sid: str, project_id: str,
                        data, metadata: MetadataContainerType) -> MetadataContainerType:
        """Create a TextGrid object

        Args:
            sid (str): Session ID
            project_id (str): Project ID
            data ([type]): the data
            metadata (MetadataContainerType): the metadata

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            MetadataContainerType: metadata for newly created object
        """
        metadata_string = self._serializer.render(metadata)
        response = super().create_resource(sid, project_id, data, metadata_string)
        return self._parser.parse(BytesIO(response.content), MetadataContainerType)

    def read_metadata(self, textgrid_uri: str, sid: Optional[str] = None) -> MetadataContainerType:
        """Read Metadata

        Args:
            textgrid_uri (str): Textgrid URI
            sid (Optional[str]): Session ID. Defaults to ''.

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
             MetadataContainerType: metadata for object
        """
        response = super().read_metadata(textgrid_uri, sid)
        return self._parser.parse(BytesIO(response.content), MetadataContainerType)

    def update_metadata(self, sid: str, textgrid_uri: str,
                        metadata: MetadataContainerType) -> MetadataContainerType:
        """Update metadata for TextGrid object

        Args:
            sid (str): Session ID
            textgrid_uri (str): Textgrid URI
            metadata (MetadataContainerType): the metadata

        Returns:
            MetadataContainerType: updated metadata
        """
        metadata_string = self._serializer.render(metadata)
        response = super().update_metadata(sid, textgrid_uri, metadata_string)
        return self._parser.parse(BytesIO(response.content), MetadataContainerType)

    def update_resource(self, sid: str, textgrid_uri: str,
                        data, metadata: MetadataContainerType) -> MetadataContainerType:
        """Update a TextGrid object

        Args:
            sid (str): Session ID
            textgrid_uri (str): Textgrid URI
            data ([type]): the data
            metadata (MetadataContainerType): the metadata

        Raises:
            TextgridCrudException: if HTTP status code >= 400   (# noqa: DAR402)

        Returns:
            MetadataContainerType: updated metadata
        """
        metadata_string = self._serializer.render(metadata)
        response = super().update_resource(sid, textgrid_uri, data, metadata_string)
        return self._parser.parse(BytesIO(response.content), MetadataContainerType)



# SPDX-FileCopyrightText: 2022 Georg-August-Universität Göttingen
#
# SPDX-License-Identifier: CC0-1.0

from dataclasses import dataclass, field
from enum import Enum
from typing import Optional

__NAMESPACE__ = "http://textgrid.info/namespaces/metadata/agent/2010"


class AgentRoleType(Enum):
    """
    The role a person or institution has for the object.

    Attributes
        ACTOR: Same as: http://id.loc.gov/vocabulary/relators/act
        ADAPTER: Same as: http://id.loc.gov/vocabulary/relators/adp
        ANALYST: Same as: http://id.loc.gov/vocabulary/relators/anl
        ANIMATOR: Same as: http://id.loc.gov/vocabulary/relators/anm
        ANNOTATOR: Same as: http://id.loc.gov/vocabulary/relators/ann
        APPLICANT: Same as: http://id.loc.gov/vocabulary/relators/app
        ARCHITECT: Same as: http://id.loc.gov/vocabulary/relators/arc
        ARRANGER: Same as: http://id.loc.gov/vocabulary/relators/arr
        ART_COPYIST: Same as: http://id.loc.gov/vocabulary/relators/acp
        ARTIST: Same as: http://id.loc.gov/vocabulary/relators/art
        ARTISTIC_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/ard
        ASSIGNEE: Same as: http://id.loc.gov/vocabulary/relators/asg
        ASSOCIATED_NAME: Same as:
            http://id.loc.gov/vocabulary/relators/asn
        ATTRIBUTED_NAME: Same as:
            http://id.loc.gov/vocabulary/relators/att
        AUCTIONEER: Same as: http://id.loc.gov/vocabulary/relators/auc
        AUTHOR: Same as: http://id.loc.gov/vocabulary/relators/aut
        AUTHOR_IN_QUOTATIONS: Same as:
            http://id.loc.gov/vocabulary/relators/aqt
        AUTHOR_OF_AFTERWORD: Same as:
            http://id.loc.gov/vocabulary/relators/aft
        AUTHOR_OF_DIALOG: Same as:
            http://id.loc.gov/vocabulary/relators/aud
        AUTHOR_OF_INTRODUCTION: Same as:
            http://id.loc.gov/vocabulary/relators/aui
        AUTHOR_OF_SCREENPLAY: Same as:
            http://id.loc.gov/vocabulary/relators/aus
        BIBLIOGRAPHIC_ANTECEDENT: Same as:
            http://id.loc.gov/vocabulary/relators/ant
        BINDER: Same as: http://id.loc.gov/vocabulary/relators/bnd
        BINDING_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/bdd
        BLURB_WRITER: Same as: http://id.loc.gov/vocabulary/relators/blw
        BOOK_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/bkd
        BOOK_PRODUCER: Same as:
            http://id.loc.gov/vocabulary/relators/bkp
        BOOK_JACKET_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/bjd
        BOOKPLATE_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/bpd
        BOOKSELLER: Same as: http://id.loc.gov/vocabulary/relators/bsl
        CALLIGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/cll
        CARTOGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/ctg
        CENSOR: Same as: http://id.loc.gov/vocabulary/relators/cns
        CINEMATOGRAPHER: Same as:
            http://id.loc.gov/vocabulary/relators/cng
        CLIENT: Same as: http://id.loc.gov/vocabulary/relators/cli
        COLLABORATOR: Same as: http://id.loc.gov/vocabulary/relators/clb
        COLLECTOR: Same as: http://id.loc.gov/vocabulary/relators/col
        COLLOTYPER: Same as: http://id.loc.gov/vocabulary/relators/clt
        COLORIST: Same as: http://id.loc.gov/vocabulary/relators/clr
        COMMENTATOR: Same as: http://id.loc.gov/vocabulary/relators/cmm
        COMMENTATOR_FOR_WRITTEN_TEXT: Same as:
            http://id.loc.gov/vocabulary/relators/cwt
        COMPILER: Same as: http://id.loc.gov/vocabulary/relators/com
        COMPLAINANT: Same as: http://id.loc.gov/vocabulary/relators/cpl
        COMPLAINANT_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/cpt
        COMPLAINANT_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/cpe
        COMPOSER: Same as: http://id.loc.gov/vocabulary/relators/cmp
        COMPOSITOR: Same as: http://id.loc.gov/vocabulary/relators/cmt
        CONCEPTOR: Same as: http://id.loc.gov/vocabulary/relators/ccp
        CONDUCTOR: Same as: http://id.loc.gov/vocabulary/relators/cnd
        CONSERVATOR: Same as: http://id.loc.gov/vocabulary/relators/con
        CONSULTANT: Same as: http://id.loc.gov/vocabulary/relators/csl
        CONSULTANT_TO_PROJECT: Same as:
            http://id.loc.gov/vocabulary/relators/csp
        CONTESTANT: Same as: http://id.loc.gov/vocabulary/relators/cos
        CONTESTANT_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/cot
        CONTESTANT_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/coe
        CONTESTEE: Same as: http://id.loc.gov/vocabulary/relators/cts
        CONTESTEE_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/ctt
        CONTESTEE_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/cte
        CONTRACTOR: Same as: http://id.loc.gov/vocabulary/relators/ctr
        CONTRIBUTOR: Same as: http://id.loc.gov/vocabulary/relators/ctb
        COPYRIGHT_CLAIMANT: Same as:
            http://id.loc.gov/vocabulary/relators/cpc
        COPYRIGHT_HOLDER: Same as:
            http://id.loc.gov/vocabulary/relators/cph
        CORRECTOR: Same as: http://id.loc.gov/vocabulary/relators/crr
        CORRESPONDENT: Same as:
            http://id.loc.gov/vocabulary/relators/crp
        COSTUME_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/cst
        COVER_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/cov
        CREATOR: Same as: http://id.loc.gov/vocabulary/relators/cre
        CURATOR_OF_AN_EXHIBITION: Same as:
            http://id.loc.gov/vocabulary/relators/cur
        DANCER: Same as: http://id.loc.gov/vocabulary/relators/dnc
        DATA_CONTRIBUTOR: Same as:
            http://id.loc.gov/vocabulary/relators/dtc
        DATA_MANAGER: Same as: http://id.loc.gov/vocabulary/relators/dtm
        DEDICATEE: Same as: http://id.loc.gov/vocabulary/relators/dte
        DEDICATOR: Same as: http://id.loc.gov/vocabulary/relators/dto
        DEFENDANT: Same as: http://id.loc.gov/vocabulary/relators/dfd
        DEFENDANT_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/dft
        DEFENDANT_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/dfe
        DEGREE_GRANTOR: Same as:
            http://id.loc.gov/vocabulary/relators/dgg
        DELINEATOR: Same as: http://id.loc.gov/vocabulary/relators/dln
        DEPICTED: Same as: http://id.loc.gov/vocabulary/relators/dpc
        DEPOSITOR: Same as: http://id.loc.gov/vocabulary/relators/dpt
        DESIGNER: Same as: http://id.loc.gov/vocabulary/relators/dsr
        DIRECTOR: Same as: http://id.loc.gov/vocabulary/relators/drt
        DISSERTANT: Same as: http://id.loc.gov/vocabulary/relators/dis
        DISTRIBUTION_PLACE: Same as:
            http://id.loc.gov/vocabulary/relators/dbp
        DISTRIBUTOR: Same as: http://id.loc.gov/vocabulary/relators/dst
        DONOR: Same as: http://id.loc.gov/vocabulary/relators/dnr
        DRAFTSMAN: Same as: http://id.loc.gov/vocabulary/relators/drm
        DUBIOUS_AUTHOR: Same as:
            http://id.loc.gov/vocabulary/relators/dub
        EDITOR: Same as: http://id.loc.gov/vocabulary/relators/edt
        ELECTRICIAN: Same as: http://id.loc.gov/vocabulary/relators/elg
        ELECTROTYPER: Same as: http://id.loc.gov/vocabulary/relators/elt
        ENGINEER: Same as: http://id.loc.gov/vocabulary/relators/eng
        ENGRAVER: Same as: http://id.loc.gov/vocabulary/relators/egr
        ETCHER: Same as: http://id.loc.gov/vocabulary/relators/etr
        EVENT_PLACE: Same as: http://id.loc.gov/vocabulary/relators/evp
        EXPERT: Same as: http://id.loc.gov/vocabulary/relators/exp
        FACSIMILIST: Same as: http://id.loc.gov/vocabulary/relators/fac
        FIELD_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/fld
        FILM_EDITOR: Same as: http://id.loc.gov/vocabulary/relators/flm
        FIRST_PARTY: Same as: http://id.loc.gov/vocabulary/relators/fpy
        FORGER: Same as: http://id.loc.gov/vocabulary/relators/frg
        FORMER_OWNER: Same as: http://id.loc.gov/vocabulary/relators/fmo
        FUNDER: Same as: http://id.loc.gov/vocabulary/relators/fnd
        GEOGRAPHIC_INFORMATION_SPECIALIST: Same as:
            http://id.loc.gov/vocabulary/relators/gis
        HONOREE: Same as: http://id.loc.gov/vocabulary/relators/hnr
        HOST: Same as: http://id.loc.gov/vocabulary/relators/hst
        ILLUMINATOR: Same as: http://id.loc.gov/vocabulary/relators/ilu
        ILLUSTRATOR: Same as: http://id.loc.gov/vocabulary/relators/ill
        INSCRIBER: Same as: http://id.loc.gov/vocabulary/relators/ins
        INSTRUMENTALIST: Same as:
            http://id.loc.gov/vocabulary/relators/itr
        INTERVIEWEE: Same as: http://id.loc.gov/vocabulary/relators/ive
        INTERVIEWER: Same as: http://id.loc.gov/vocabulary/relators/ivr
        INVENTOR: Same as: http://id.loc.gov/vocabulary/relators/inv
        LABORATORY: Same as: http://id.loc.gov/vocabulary/relators/lbr
        LABORATORY_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/ldr
        LEAD: Same as: http://id.loc.gov/vocabulary/relators/led
        LANDSCAPE_ARCHITECT: Same as:
            http://id.loc.gov/vocabulary/relators/lsa
        LENDER: Same as: http://id.loc.gov/vocabulary/relators/len
        LIBELANT: Same as: http://id.loc.gov/vocabulary/relators/lil
        LIBELANT_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/lit
        LIBELANT_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/lie
        LIBELEE: Same as: http://id.loc.gov/vocabulary/relators/lel
        LIBELEE_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/let
        LIBELEE_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/lee
        LIBRETTIST: Same as: http://id.loc.gov/vocabulary/relators/lbt
        LICENSEE: Same as: http://id.loc.gov/vocabulary/relators/lse
        LICENSOR: Same as: http://id.loc.gov/vocabulary/relators/lso
        LIGHTING_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/lgd
        LITHOGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/ltg
        LYRICIST: Same as: http://id.loc.gov/vocabulary/relators/lyr
        MANUFACTURER: Same as: http://id.loc.gov/vocabulary/relators/mfr
        MARBLER: Same as: http://id.loc.gov/vocabulary/relators/mrb
        MARKUP_EDITOR: Same as:
            http://id.loc.gov/vocabulary/relators/mrk
        METADATA_CONTACT: Same as:
            http://id.loc.gov/vocabulary/relators/mdc
        METALENGRAVER: Same as:
            http://id.loc.gov/vocabulary/relators/mte
        MODERATOR: Same as: http://id.loc.gov/vocabulary/relators/mod
        MONITOR: Same as: http://id.loc.gov/vocabulary/relators/mon
        MUSIC_COPYIST: Same as:
            http://id.loc.gov/vocabulary/relators/mcp
        MUSICAL_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/msd
        MUSICIAN: Same as: http://id.loc.gov/vocabulary/relators/mus
        NARRATOR: Same as: http://id.loc.gov/vocabulary/relators/nrt
        OPPONENT: Same as: http://id.loc.gov/vocabulary/relators/opn
        ORGANIZER_OF_MEETING: Same as:
            http://id.loc.gov/vocabulary/relators/orm
        ORIGINATOR: Same as: http://id.loc.gov/vocabulary/relators/org
        OTHER: Same as: http://id.loc.gov/vocabulary/relators/oth
        OWNER: Same as: http://id.loc.gov/vocabulary/relators/own
        PAPERMAKER: Same as: http://id.loc.gov/vocabulary/relators/ppm
        PATENT_APPLICANT: Same as:
            http://id.loc.gov/vocabulary/relators/pta
        PATENT_HOLDER: Same as:
            http://id.loc.gov/vocabulary/relators/pth
        PATRON: Same as: http://id.loc.gov/vocabulary/relators/pat
        PERFORMER: Same as: http://id.loc.gov/vocabulary/relators/prf
        PERMITTING_AGENCY: Same as:
            http://id.loc.gov/vocabulary/relators/pma
        PHOTOGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/pht
        PLAINTIFF: Same as: http://id.loc.gov/vocabulary/relators/ptf
        PLAINTIFF_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/ptt
        PLAINTIFF_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/pte
        PLATEMAKER: Same as: http://id.loc.gov/vocabulary/relators/plt
        PRINTER: Same as: http://id.loc.gov/vocabulary/relators/prt
        PRINTER_OF_PLATES: Same as:
            http://id.loc.gov/vocabulary/relators/pop
        PRINTMAKER: Same as: http://id.loc.gov/vocabulary/relators/prm
        PROCESS_CONTACT: Same as:
            http://id.loc.gov/vocabulary/relators/prc
        PRODUCER: Same as: http://id.loc.gov/vocabulary/relators/pro
        PRODUCTION_MANAGER: Same as:
            http://id.loc.gov/vocabulary/relators/pmm
        PRODUCTION_PERSONNEL: Same as:
            http://id.loc.gov/vocabulary/relators/prd
        PROGRAMMER: Same as: http://id.loc.gov/vocabulary/relators/prg
        PROJECT_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/pdr
        PROOFREADER: Same as: http://id.loc.gov/vocabulary/relators/pfr
        PUBLICATION_PLACE: Same as:
            http://id.loc.gov/vocabulary/relators/pup
        PUBLISHER: Same as: http://id.loc.gov/vocabulary/relators/pbl
        PUBLISHING_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/pbd
        PUPPETEER: Same as: http://id.loc.gov/vocabulary/relators/ppt
        RECIPIENT: Same as: http://id.loc.gov/vocabulary/relators/rcp
        RECORDING_ENGINEER: Same as:
            http://id.loc.gov/vocabulary/relators/rce
        REDACTOR: Same as: http://id.loc.gov/vocabulary/relators/red
        RENDERER: Same as: http://id.loc.gov/vocabulary/relators/ren
        REPORTER: Same as: http://id.loc.gov/vocabulary/relators/rpt
        REPOSITORY: Same as: http://id.loc.gov/vocabulary/relators/rps
        RESEARCH_TEAM_HEAD: Same as:
            http://id.loc.gov/vocabulary/relators/rth
        RESEARCH_TEAM_MEMBER: Same as:
            http://id.loc.gov/vocabulary/relators/rtm
        RESEARCHER: Same as: http://id.loc.gov/vocabulary/relators/res
        RESPONDENT: Same as: http://id.loc.gov/vocabulary/relators/rsp
        RESPONDENT_APPELLANT: Same as:
            http://id.loc.gov/vocabulary/relators/rst
        RESPONDENT_APPELLEE: Same as:
            http://id.loc.gov/vocabulary/relators/rse
        RESPONSIBLE_PARTY: Same as:
            http://id.loc.gov/vocabulary/relators/rpy
        RESTAGER: Same as: http://id.loc.gov/vocabulary/relators/rsg
        REVIEWER: Same as: http://id.loc.gov/vocabulary/relators/rev
        RUBRICATOR: Same as: http://id.loc.gov/vocabulary/relators/rbr
        SCENARIST: Same as: http://id.loc.gov/vocabulary/relators/sce
        SCIENTIFIC_ADVISOR: Same as:
            http://id.loc.gov/vocabulary/relators/sad
        SCRIBE: Same as: http://id.loc.gov/vocabulary/relators/scr
        SCULPTOR: Same as: http://id.loc.gov/vocabulary/relators/scl
        SECOND_PARTY: Same as: http://id.loc.gov/vocabulary/relators/spy
        SECRETARY: Same as: http://id.loc.gov/vocabulary/relators/sec
        SETDESIGNER: Same as: http://id.loc.gov/vocabulary/relators/std
        SIGNER: Same as: http://id.loc.gov/vocabulary/relators/sgn
        SINGER: Same as: http://id.loc.gov/vocabulary/relators/sng
        SOUND_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/sds
        SPEAKER: Same as: http://id.loc.gov/vocabulary/relators/spk
        SPONSOR: Same as: http://id.loc.gov/vocabulary/relators/spn
        STAGE_MANAGER: Same as:
            http://id.loc.gov/vocabulary/relators/stm
        STANDARDS_BODY: Same as:
            http://id.loc.gov/vocabulary/relators/stn
        STEREOTYPER: Same as: http://id.loc.gov/vocabulary/relators/str
        STORYTELLER: Same as: http://id.loc.gov/vocabulary/relators/stl
        SUPPORTING_HOST: Same as:
            http://id.loc.gov/vocabulary/relators/sht
        SURVEYOR: Same as: http://id.loc.gov/vocabulary/relators/srv
        TEACHER: Same as: http://id.loc.gov/vocabulary/relators/tch
        TECHNICAL_DIRECTOR: Same as:
            http://id.loc.gov/vocabulary/relators/tcd
        THESIS_ADVISOR: Same as:
            http://id.loc.gov/vocabulary/relators/ths
        TRANSCRIBER: Same as: http://id.loc.gov/vocabulary/relators/trc
        TRANSLATOR: Same as: http://id.loc.gov/vocabulary/relators/trl
        TYPE_DESIGNER: Same as:
            http://id.loc.gov/vocabulary/relators/tyd
        TYPOGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/tyg
        UNIVERSITY_PLACE: Same as:
            http://id.loc.gov/vocabulary/relators/uvp
        VIDEOGRAPHER: Same as: http://id.loc.gov/vocabulary/relators/vdg
        VOCALIST: Same as: http://id.loc.gov/vocabulary/relators/voc
        WITNESS: Same as: http://id.loc.gov/vocabulary/relators/wit
        WOOD_ENGRAVER: Same as:
            http://id.loc.gov/vocabulary/relators/wde
        WOODCUTTER: Same as: http://id.loc.gov/vocabulary/relators/wdc
        WRITER_OF_ACCOMPANYING_MATERIAL: Same as:
            http://id.loc.gov/vocabulary/relators/wam
    """
    ACTOR = "actor"
    ADAPTER = "adapter"
    ANALYST = "analyst"
    ANIMATOR = "animator"
    ANNOTATOR = "annotator"
    APPLICANT = "applicant"
    ARCHITECT = "architect"
    ARRANGER = "arranger"
    ART_COPYIST = "artCopyist"
    ARTIST = "artist"
    ARTISTIC_DIRECTOR = "artisticDirector"
    ASSIGNEE = "assignee"
    ASSOCIATED_NAME = "associatedName"
    ATTRIBUTED_NAME = "attributedName"
    AUCTIONEER = "auctioneer"
    AUTHOR = "author"
    AUTHOR_IN_QUOTATIONS = "authorInQuotations"
    AUTHOR_OF_AFTERWORD = "authorOfAfterword"
    AUTHOR_OF_DIALOG = "authorOfDialog"
    AUTHOR_OF_INTRODUCTION = "authorOfIntroduction"
    AUTHOR_OF_SCREENPLAY = "authorOfScreenplay"
    BIBLIOGRAPHIC_ANTECEDENT = "bibliographicAntecedent"
    BINDER = "binder"
    BINDING_DESIGNER = "bindingDesigner"
    BLURB_WRITER = "blurbWriter"
    BOOK_DESIGNER = "bookDesigner"
    BOOK_PRODUCER = "bookProducer"
    BOOK_JACKET_DESIGNER = "bookJacketDesigner"
    BOOKPLATE_DESIGNER = "bookplateDesigner"
    BOOKSELLER = "bookseller"
    CALLIGRAPHER = "calligrapher"
    CARTOGRAPHER = "cartographer"
    CENSOR = "censor"
    CINEMATOGRAPHER = "cinematographer"
    CLIENT = "client"
    COLLABORATOR = "collaborator"
    COLLECTOR = "collector"
    COLLOTYPER = "collotyper"
    COLORIST = "colorist"
    COMMENTATOR = "commentator"
    COMMENTATOR_FOR_WRITTEN_TEXT = "commentatorForWrittenText"
    COMPILER = "compiler"
    COMPLAINANT = "complainant"
    COMPLAINANT_APPELLANT = "complainantAppellant"
    COMPLAINANT_APPELLEE = "complainantAppellee"
    COMPOSER = "composer"
    COMPOSITOR = "compositor"
    CONCEPTOR = "conceptor"
    CONDUCTOR = "conductor"
    CONSERVATOR = "conservator"
    CONSULTANT = "consultant"
    CONSULTANT_TO_PROJECT = "consultantToProject"
    CONTESTANT = "contestant"
    CONTESTANT_APPELLANT = "contestantAppellant"
    CONTESTANT_APPELLEE = "contestantAppellee"
    CONTESTEE = "contestee"
    CONTESTEE_APPELLANT = "contesteeAppellant"
    CONTESTEE_APPELLEE = "contesteeAppellee"
    CONTRACTOR = "contractor"
    CONTRIBUTOR = "contributor"
    COPYRIGHT_CLAIMANT = "copyrightClaimant"
    COPYRIGHT_HOLDER = "copyrightHolder"
    CORRECTOR = "corrector"
    CORRESPONDENT = "correspondent"
    COSTUME_DESIGNER = "costumeDesigner"
    COVER_DESIGNER = "coverDesigner"
    CREATOR = "creator"
    CURATOR_OF_AN_EXHIBITION = "curatorOfAnExhibition"
    DANCER = "dancer"
    DATA_CONTRIBUTOR = "dataContributor"
    DATA_MANAGER = "dataManager"
    DEDICATEE = "dedicatee"
    DEDICATOR = "dedicator"
    DEFENDANT = "defendant"
    DEFENDANT_APPELLANT = "defendantAppellant"
    DEFENDANT_APPELLEE = "defendantAppellee"
    DEGREE_GRANTOR = "degreeGrantor"
    DELINEATOR = "delineator"
    DEPICTED = "depicted"
    DEPOSITOR = "depositor"
    DESIGNER = "designer"
    DIRECTOR = "director"
    DISSERTANT = "dissertant"
    DISTRIBUTION_PLACE = "distributionPlace"
    DISTRIBUTOR = "distributor"
    DONOR = "donor"
    DRAFTSMAN = "draftsman"
    DUBIOUS_AUTHOR = "dubiousAuthor"
    EDITOR = "editor"
    ELECTRICIAN = "electrician"
    ELECTROTYPER = "electrotyper"
    ENGINEER = "engineer"
    ENGRAVER = "engraver"
    ETCHER = "etcher"
    EVENT_PLACE = "eventPlace"
    EXPERT = "expert"
    FACSIMILIST = "facsimilist"
    FIELD_DIRECTOR = "fieldDirector"
    FILM_EDITOR = "filmEditor"
    FIRST_PARTY = "firstParty"
    FORGER = "forger"
    FORMER_OWNER = "formerOwner"
    FUNDER = "funder"
    GEOGRAPHIC_INFORMATION_SPECIALIST = "geographicInformationSpecialist"
    HONOREE = "honoree"
    HOST = "host"
    ILLUMINATOR = "illuminator"
    ILLUSTRATOR = "illustrator"
    INSCRIBER = "inscriber"
    INSTRUMENTALIST = "instrumentalist"
    INTERVIEWEE = "interviewee"
    INTERVIEWER = "interviewer"
    INVENTOR = "inventor"
    LABORATORY = "laboratory"
    LABORATORY_DIRECTOR = "laboratoryDirector"
    LEAD = "lead"
    LANDSCAPE_ARCHITECT = "landscapeArchitect"
    LENDER = "lender"
    LIBELANT = "libelant"
    LIBELANT_APPELLANT = "libelantAppellant"
    LIBELANT_APPELLEE = "libelantAppellee"
    LIBELEE = "libelee"
    LIBELEE_APPELLANT = "libeleeAppellant"
    LIBELEE_APPELLEE = "libeleeAppellee"
    LIBRETTIST = "librettist"
    LICENSEE = "licensee"
    LICENSOR = "licensor"
    LIGHTING_DESIGNER = "lightingDesigner"
    LITHOGRAPHER = "lithographer"
    LYRICIST = "lyricist"
    MANUFACTURER = "manufacturer"
    MARBLER = "marbler"
    MARKUP_EDITOR = "markupEditor"
    METADATA_CONTACT = "metadataContact"
    METALENGRAVER = "metalengraver"
    MODERATOR = "moderator"
    MONITOR = "monitor"
    MUSIC_COPYIST = "musicCopyist"
    MUSICAL_DIRECTOR = "musicalDirector"
    MUSICIAN = "musician"
    NARRATOR = "narrator"
    OPPONENT = "opponent"
    ORGANIZER_OF_MEETING = "organizerOfMeeting"
    ORIGINATOR = "originator"
    OTHER = "other"
    OWNER = "owner"
    PAPERMAKER = "papermaker"
    PATENT_APPLICANT = "patentApplicant"
    PATENT_HOLDER = "patentHolder"
    PATRON = "patron"
    PERFORMER = "performer"
    PERMITTING_AGENCY = "permittingAgency"
    PHOTOGRAPHER = "photographer"
    PLAINTIFF = "plaintiff"
    PLAINTIFF_APPELLANT = "plaintiffAppellant"
    PLAINTIFF_APPELLEE = "plaintiffAppellee"
    PLATEMAKER = "platemaker"
    PRINTER = "printer"
    PRINTER_OF_PLATES = "printerOfPlates"
    PRINTMAKER = "printmaker"
    PROCESS_CONTACT = "processContact"
    PRODUCER = "producer"
    PRODUCTION_MANAGER = "productionManager"
    PRODUCTION_PERSONNEL = "productionPersonnel"
    PROGRAMMER = "programmer"
    PROJECT_DIRECTOR = "projectDirector"
    PROOFREADER = "proofreader"
    PUBLICATION_PLACE = "publicationPlace"
    PUBLISHER = "publisher"
    PUBLISHING_DIRECTOR = "publishingDirector"
    PUPPETEER = "puppeteer"
    RECIPIENT = "recipient"
    RECORDING_ENGINEER = "recordingEngineer"
    REDACTOR = "redactor"
    RENDERER = "renderer"
    REPORTER = "reporter"
    REPOSITORY = "repository"
    RESEARCH_TEAM_HEAD = "researchTeamHead"
    RESEARCH_TEAM_MEMBER = "researchTeamMember"
    RESEARCHER = "researcher"
    RESPONDENT = "respondent"
    RESPONDENT_APPELLANT = "respondentAppellant"
    RESPONDENT_APPELLEE = "respondentAppellee"
    RESPONSIBLE_PARTY = "responsibleParty"
    RESTAGER = "restager"
    REVIEWER = "reviewer"
    RUBRICATOR = "rubricator"
    SCENARIST = "scenarist"
    SCIENTIFIC_ADVISOR = "scientificAdvisor"
    SCRIBE = "scribe"
    SCULPTOR = "sculptor"
    SECOND_PARTY = "secondParty"
    SECRETARY = "secretary"
    SETDESIGNER = "setdesigner"
    SIGNER = "signer"
    SINGER = "singer"
    SOUND_DESIGNER = "soundDesigner"
    SPEAKER = "speaker"
    SPONSOR = "sponsor"
    STAGE_MANAGER = "stageManager"
    STANDARDS_BODY = "standardsBody"
    STEREOTYPER = "stereotyper"
    STORYTELLER = "storyteller"
    SUPPORTING_HOST = "supportingHost"
    SURVEYOR = "surveyor"
    TEACHER = "teacher"
    TECHNICAL_DIRECTOR = "technicalDirector"
    THESIS_ADVISOR = "thesisAdvisor"
    TRANSCRIBER = "transcriber"
    TRANSLATOR = "translator"
    TYPE_DESIGNER = "typeDesigner"
    TYPOGRAPHER = "typographer"
    UNIVERSITY_PLACE = "universityPlace"
    VIDEOGRAPHER = "videographer"
    VOCALIST = "vocalist"
    WITNESS = "Witness"
    WOOD_ENGRAVER = "woodEngraver"
    WOODCUTTER = "woodcutter"
    WRITER_OF_ACCOMPANYING_MATERIAL = "writerOfAccompanyingMaterial"


@dataclass
class PersonType:
    class Meta:
        name = "personType"

    value: str = field(
        default="",
        metadata={
            "required": True,
        }
    )
    id: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
        }
    )
    is_corporate_body: bool = field(
        default=False,
        metadata={
            "name": "isCorporateBody",
            "type": "Attribute",
        }
    )


@dataclass
class AgentType:
    class Meta:
        name = "agentType"

    value: str = field(
        default="",
        metadata={
            "required": True,
        }
    )
    role: Optional[AgentRoleType] = field(
        default=None,
        metadata={
            "type": "Attribute",
        }
    )
    id: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
        }
    )
